==> release.12.2.28.1.tgz: cgi-cpn/prontus_dropbox_backup.pl <==
#!/usr/bin/perl
# --------------------------------------------------------------
# Respaldo de archivos prontus en Dropbox.
# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

END {
    &tareas_salida();
}
use sigtrap 'handler' => \&signal_catch, 'INT';
use sigtrap 'handler' => \&signal_catch, 'TERM';
use sigtrap 'handler' => \&signal_catch, 'KILL';

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Digest::SHA;
use Email::MIME;
use Email::Sender::Simple;
use Email::Sender::Transport::SMTP;
use Email::Simple;
use Encode;
use File::Copy;
use File::LibMagic;
use HTTP::Request;
use IO::All;
use JSON;
use LWP::UserAgent;
use Time::Local;
use glib_fildir_03;
use lib_dropbox;
use lib_mail;
use lib_utils;
##/libs##

my %FORM;
my $DIR_SEMAF;

main: {
    lib_utils::setUtf8();
    $FORM{'prontus_id'} = $ARGV[0];
    if ($FORM{'prontus_id'} eq '') {
        print "Error en parametros\n";
    }

    # Carga variables de configuracion de prontus.
    my $msg_load = lib_utils::load_config_by_id($FORM{'prontus_id'});
    if ($msg_load ne '') {
        print STDERR "Error: $msg_load\n";
        print "Error: $msg_load\n";
        exit;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX'} eq 'NO'
       || $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'} eq '') {
        # No se hace nada.
        print STDERR "Dropbox deshabilitado o no hay refresh token: [$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'}]\n";
        exit;
    }

    &lib_dropbox::printDebug("[$$] Inicio\n");

    $DIR_SEMAF = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/semaforos";
    glib_fildir_03::check_dir($DIR_SEMAF);

    if (-f "$DIR_SEMAF/dropbox_backup.lck") {
        my $mtime = (stat("$DIR_SEMAF/dropbox_backup.lck"))[9];
        my $now = time;
        my $diff = $now - $mtime;

        if ($diff > 7200) { # 2 hrs.
            &lib_dropbox::printDebug("[$$] Semaforo muy antiguo, eliminando...");
            unlink "$DIR_SEMAF/dropbox_backup.lck";

            my $res = `ps auxww |grep 'prontus_dropbox_backup.pl $prontus_varglb::PRONTUS_ID' | grep -v grep | wc -l`;

            if ($res) {
                system('kill -9 `ps -auxww | grep \'prontus_dropbox_backup.pl ' . $prontus_varglb::PRONTUS_ID . '\' | grep -v grep | awk \'{print $2}\' | grep -v ' . $$ . '`');
            }
        }
    }

    glib_fildir_03::write_file("$DIR_SEMAF/dropbox_backup.lck", $$);

    # Inicializa modulo dropbox.
    &lib_dropbox::init($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'});

    # Revisa si hay archivos.
    my $dir_dropbox = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_DBM'}/dropbox";
    my ($file);
    my %days; # Dias a procesar (un dia especial es 'port', que procesa las portadas).

    my @files = glob("$dir_dropbox/*.txt");
    while (scalar @files > 0) {
        &lib_dropbox::printDebug("[$$] Buscando archivos...\n");
        %days = ();
        while (defined($file = shift @files)) {
            &lib_dropbox::printDebug("[$$] Procesando [$file]\n");
            next if (!-s $file);
            &procesar_archivo($file,\%days);
            # print STDERR "[$$] Se termino de procesar [$file].\n";

        }
        # Respalda los contenidos correspondientes.
        foreach my $recurso (keys %days) {
            &lib_dropbox::printDebug("[$$] Procesando recurso [$recurso]\n");
            if ($recurso eq 'port') {
                &procesar_port();
            } else {
                &procesa_art($recurso);
            }
            # Actualiza fecha modificacion semaforo, para que no lo vayan a matar!
            utime time, time, "$DIR_SEMAF/dropbox_backup.lck";
        }

        @files = glob("$dir_dropbox/*.txt");
    }

    # Respalda la taxonomia.
    #&procesa_tax();

    &lib_dropbox::printDebug("[$$] Fin.\n");

}

# ----------------------------------------------------------------------------- #
# tareas que se deben realizar al terminar el script
sub tareas_salida {
    unlink "$DIR_SEMAF/dropbox_backup.lck";
}

# ---------------------------------------------------------------
# funcion para capturar las señales INT y TERM y logearlas
sub signal_catch {
    print STDERR  "Terminado por signal @_\n";
    exit(0);
}

# ----------------------------------------------------------------------------- #
sub procesar_archivo {
    my ($file,$days) = @_;

    open (my $FILE, '<', $file);
    foreach my $linea (<$FILE>) {
        chomp($linea);
        my ($tipo, $recurso, $param1, $param2, $param3) = split(';', $linea);
        &lib_dropbox::printDebug("[$$] tipo[$tipo] recurso[$recurso]\n");
        if ($tipo eq 'art') {
            # Reduce recurso a un dia especifico.
            $recurso = substr($recurso,0,8);
            $$days{$recurso} = 1;
        } elsif ($tipo eq 'port') {
            $$days{'port'} = 1;
        }
    }
    close $FILE;

    unlink $file;
}

# ----------------------------------------------------------------------------- #
# Sincroniza los directorios de portadas completos.
sub procesar_port {
    &recurseDirs("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/edic",
                                            "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/edic");
}

# ----------------------------------------------------------------------------- #
# Sincroniza el directorio del dia del articulo completo.
sub procesa_art {
    my $ts = $_[0];
    my $dir_fecha;

    if ($ts =~ /(\d{8})/) {
        $dir_fecha = $1; # solo fecha.
        &recurseDirs("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dir_fecha",
                                                "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dir_fecha");
        # Se incluye la media externa.
        &recurseDirs("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EXMEDIA/$dir_fecha",
                                                "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EXMEDIA/$dir_fecha");
    }
}

# ----------------------------------------------------------------------------- #
# Sincroniza el directorio de la taxonomia.
sub procesa_tax {
    # Incluye taxonomia.
    &recurseDirs("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/cache/taxonomia/pags",
                "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/cache/taxonomia/pags");
}

# ----------------------------------------------------------------------------- #
# Recorre los directorios recursivamente, subiendo a Dropbox los archivos que no
# existen o que tienen un peso distinto.
sub recurseDirs {
    my($source,$destination) = @_;
    my($path,$esdir,$size,$pid,$res);
    my ($fileTime,$numEntries,$aux,$localHash,$dropBoxHash,$debeSubir);
    my(@entries,@dropboxEntries);
    my %dropboxFileSizes;
    my %dropboxFileTimes;
    my %dropboxFileHashes;

    # Si el directorio es cpan/data/search o es /cache o /bak, sale sin hacer nada.
    if (($source =~ /cpan\/data\/search/) || ($source =~ /\/cache$/) || ($source =~ /\/bak$/)) {
        # print "    Banned Dir: $source\n"; # debug
        return;
    }

    opendir(DIR, $source) || die "Can't opendir " . $source . $!;
    @entries = readdir(DIR);
    closedir DIR;

    # Si el directorio esta vacio, sale sin hacer nada.
    $numEntries = scalar(@entries);
    if ($numEntries <= 2) {
        # print "    Empty Dir: $source\n"; # debug
        return;
    }

    &lib_dropbox::printDebug("[$$] Inicio: $source [$numEntries] -> $destination\n");

    # Lee las entradas de dropbox en el destino. <path>\t<es dir>\t<tamano en bytes>
    @dropboxEntries = lib_dropbox::getDir($destination);
    if ($lib_dropbox::ERROR) {
        # intentamos regenerar el token on the fly.
        if ($lib_dropbox::ERROR =~ /401 Unauthorized.*?invalid_access_token/) {
            $lib_dropbox::ATOKEN = '';
            $lib_dropbox::ATOKEN = &lib_dropbox::getAccessToken($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CLIENT_ID'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CLIENT_SECRET'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'});
            &lib_dropbox::printDebug("Auth token: $lib_dropbox::ATOKEN\n");
            if (!$lib_dropbox::ATOKEN) {
                return;
            }
        } else {
            return;
        }
    }
    # Obtiene los tamanos de archivo. Si es un directorio asigna el tamano -1.
    %dropboxFileSizes = ();
    %dropboxFileTimes = ();
    # print "    Listado de $destination en dropbox:\n";
    foreach my $dropBoxEntry (@dropboxEntries) {
        ($path,$esdir,$size,$fileTime,$dropBoxHash) = split(/\t/,$dropBoxEntry);
        # Elimina los directorios superiores del path.
        if ($path =~ /\/([^\/]+)$/) {
            $path = $1;
        } else {
            $path = '/';
        }
        # Asigna el tamano.
        if ($esdir) {
            $dropboxFileSizes{$path} = -1;
        } else {
            $dropboxFileSizes{$path} = $size;
        }
        # Asigna el tiempo de modificacion en Dropbox.
        $dropboxFileTimes{$path} = lib_dropbox::iso8601ToEpoch($fileTime);
        $dropboxFileHashes{$path} = $dropBoxHash;
        # print "    $path $size $fileTime\n"; # debug
    }

    # Recorre los archivos y subdirectorios.
    foreach my $entry (sort @entries) {
        # Si es un directorio, entonces aplica de nuevo la rutina en forma recursiva.
        # No copia los directorios que empiezan con un punto.
        if ((-d "$source/$entry") && ($entry !~ /^\./)) {
            &recurseDirs("$source/$entry","$destination/$entry");
        }

        if (-f "$source/$entry") {
            # No considera los archivos que contengan 'preview.' en su nombre,
            # o sean nombres baneados por dropbox.
            $aux = lc $entry;
            if (($aux =~ /preview\./) || ($aux eq 'desktop.ini') || ($aux eq 'thumbs.db')
                 || ($aux eq '.ds_store') || ($aux eq "icon\r") || ($aux =~ /^\.dropbox/)) {
                # print "  Me salto preview: $source/$entry\n";
                next;
            }
            # Si los tamanos son cero arriba y abajo, no hace nada.
            if (($dropboxFileSizes{$entry} == 0) && ((-s "$source/$entry") == 0)) {
                next;
            }
            $debeSubir = 0;
            # Si no existe en dropbox o si los tamanos son distintos, lo sube.
            if ( (! $dropboxFileSizes{$entry}) || ( $dropboxFileSizes{$entry} != -s "$source/$entry") ) {
                $debeSubir = 1;
            }
            ($dropBoxHash,$localHash) = ('','');
            # Si la fecha local es mas reciente que en dropbox, verifica hashes y sube si corresponde.
            if (($debeSubir == 0) && ( $dropboxFileTimes{$entry} < (stat "$source/$entry")[9] )) {
                # Compara los hashes por si los contenidos son identicos (suele suceder...).
                $dropBoxHash = $dropboxFileHashes{$entry};
                $localHash = lib_dropbox::computeContentHash("$source/$entry");
                if ($localHash ne $dropBoxHash) {
                    $debeSubir = 1;
                }
            }
            if ($debeSubir == 1) {
                &lib_dropbox::printDebug("[$$] Sube: $source/$entry d[", $dropboxFileSizes{$entry});
                &lib_dropbox::printDebug("] s[", (-s "$source/$entry") , "] t[" ,((stat "$source/$entry")[9] - $dropboxFileTimes{$entry}));
                &lib_dropbox::printDebug("] d[$dropBoxHash] s[$localHash]\n");
                # Hace la sincronizacion.
                $res = 0;
                while ($res == 0) {
                    $res = lib_dropbox::putFile("$source/$entry","$destination/$entry");
                    if ($res < 0) {
                        &lib_dropbox::printError("[$$] Falla al subir [$res]: $source/$entry\n");
                        if ($res == -3) {
                            &lib_dropbox::printError("[$$] Falló la autenticación con Dropbox. Reintentar todo.\n");
                            die;
                        }
                    } else {
                        &lib_dropbox::printDebug("[$$] Res OK [$res]: $source/$entry\n");
                    }
                }
            }
        }
    }
    if ($lib_dropbox::ERROR ne '') {
        &enviar_mail_error();
    }
}

# ----------------------------------------------------------------------------- #
sub epochToFechaHora {
    my $epoch = $_[0];
    my @utc = gmtime($epoch);
    my $fechahora = sprintf('%04d/%02d/%02d %02d:%02d:%02d',(1900 + $utc[5]),(1 + $utc[4]),$utc[3],$utc[2],$utc[1],$utc[0]);
}

# Envía un email con los errores encontrados
# @param string $email el destinatario
# @param string $texto el mensaje.
# @return el resultado de lib_mail::enviar_mail()
sub enviar_mail_error {
    my $email = 'area_prontus@altavoz.net';
    my $from = 'prontus@' . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'};
    my $replyto_name = 'Prontus CMS';
    my $replyto_email = $email;
    my $server = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
    if ($server =~ /^[0-9]+\.[0-9]+\./) {
        my $dir_server_ofusc = $prontus_varglb::DIR_SERVER;
        $dir_server_ofusc =~ s/\//-/g;
        $dir_server_ofusc =~ s/^[- ]+//g;
        $server .= " ($dir_server_ofusc)";
    }
    my $asunto = "[prontus][dropbox] Error cargando archivos de $server - prontus_id: $prontus_varglb::PRONTUS_ID";

    my $texto = $lib_dropbox::ERROR;
    my $htmldoc = '';
    my $attach = '';
    my $url = '';
    my $dir_attach = '';
    my $smtp = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_SMTP'};

    lib_mail::enviar_mail($email, $from, $replyto_name, $replyto_email, $asunto, $texto, $htmldoc, $attach, $url, $dir_attach, $smtp);
}

==> release.12.3.1.2.tgz: cgi-cpn/prontus_dropbox_backup.pl <==
#!/usr/bin/perl
# --------------------------------------------------------------
# Respaldo de archivos prontus en Dropbox.
# --------------------------------------------------------------
# Prontus CMS
# http://www.prontus.cl
# by Altavoz.net
#
# licensed under LGPL license.
# http://www.prontus.cl/license.html
# --------------------------------------------------------------
use strict;
use utf8;

BEGIN {
    use FindBin qw($Bin);
    my $pathLibsProntus = $Bin;
    unshift(@INC,$pathLibsProntus);
}

END {
    &tareas_salida();
}
use sigtrap 'handler' => \&signal_catch, 'INT';
use sigtrap 'handler' => \&signal_catch, 'TERM';
use sigtrap 'handler' => \&signal_catch, 'KILL';

##libs##
use lib_stdlog;
lib_stdlog::set_stdlog($0, 51200);
use prontus_varglb;
prontus_varglb::init();

use Digest::SHA;
use Email::MIME;
use Email::Sender::Simple;
use Email::Sender::Transport::SMTP;
use Email::Simple;
use Encode;
use File::Copy;
use File::LibMagic;
use HTTP::Request;
use IO::All;
use JSON;
use LWP::UserAgent;
use Time::Local;
use glib_fildir_03;
use glib_hrfec_03;
use lib_dropbox;
use lib_lang;
use lib_mail;
use lib_utils;
##/libs##

my %FORM;
my $DIR_SEMAF;

main: {
    lib_utils::setUp();
    $FORM{'prontus_id'} = $ARGV[0];
    if ($FORM{'prontus_id'} eq '') {
        print lib_lang::get_message('global_wrong_arguments') . "\n";
    }

    # Carga variables de configuracion de prontus.
    my $msg_load = lib_utils::load_config_by_id($FORM{'prontus_id'});
    if ($msg_load ne '') {
        print STDERR "[$$] $msg_load\n";
        print "Error: $msg_load\n";
        exit;
    }

    if ($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX'} eq 'NO'
       || $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'} eq '') {
        # No se hace nada.
        print STDERR lib_lang::get_message('utils_dropbox_disabled') . "\n";
        exit;
    }

    print STDERR "[$$] Start\n";

    $DIR_SEMAF = "$prontus_varglb::DIR_SERVER/$prontus_varglb::PRONTUS_ID/cpan/data/semaforos";
    glib_fildir_03::check_dir($DIR_SEMAF);

    if (-f "$DIR_SEMAF/dropbox_backup.lck") {
        my $mtime = (stat("$DIR_SEMAF/dropbox_backup.lck"))[9];
        my $now = time;
        my $diff = $now - $mtime;

        if ($diff > 7200) { # 2 hrs.
            print STDERR "[$$] Semaphore too old, deleting...";
            unlink "$DIR_SEMAF/dropbox_backup.lck";

            my $res = `ps auxww |grep 'prontus_dropbox_backup.pl $prontus_varglb::PRONTUS_ID' | grep -v grep | wc -l`;

            if ($res) {
                system('kill -9 `ps -auxww | grep \'prontus_dropbox_backup.pl ' . $prontus_varglb::PRONTUS_ID . '\' | grep -v grep | awk \'{print $2}\' | grep -v ' . $$ . '`');
            }
        }
    }

    glib_fildir_03::write_file("$DIR_SEMAF/dropbox_backup.lck", $$);

    # Inicializa modulo dropbox.
    &lib_dropbox::init($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'});

    # Revisa si hay archivos.
    my $dir_dropbox = "$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_CPAN_DATA'}/dropbox";
    my ($file);
    my %days; # Dias a procesar (un dia especial es 'port', que procesa las portadas).

    my @files = glob("$dir_dropbox/*.txt");
    while (scalar @files > 0) {
        print STDERR "[$$] Searching for files...\n";
        %days = ();
        while (defined($file = shift @files)) {
            print STDERR "[$$] Processing [$file]\n";
            next if (!-s $file);
            &procesar_archivo($file,\%days);
            # print STDERR "[$$] Se termino de procesar [$file].\n";

        }
        # Respalda los contenidos correspondientes.
        foreach my $recurso (keys %days) {
            print STDERR "[$$] Processing resource [$recurso]\n";
            if ($recurso eq 'port') {
                &procesar_port();
            } else {
                &procesa_art($recurso);
            }
            # Actualiza fecha modificacion semaforo, para que no lo vayan a matar!
            utime time, time, "$DIR_SEMAF/dropbox_backup.lck";
        }

        @files = glob("$dir_dropbox/*.txt");
    }

    # Respalda la taxonomia.
    #&procesa_tax();

    print STDERR "[$$] End.\n";

}

# ----------------------------------------------------------------------------- #
# tareas que se deben realizar al terminar el script
sub tareas_salida {
    unlink "$DIR_SEMAF/dropbox_backup.lck";
}

# ---------------------------------------------------------------
# funcion para capturar las señales INT y TERM y logearlas
sub signal_catch {
    print STDERR lib_lang::get_interpolated_message('global_ended_by_signal', "@_");
    exit(0);
}

# ----------------------------------------------------------------------------- #
sub procesar_archivo {
    my ($file,$days) = @_;

    open (my $FILE, '<', $file);
    foreach my $linea (<$FILE>) {
        chomp($linea);
        my ($tipo, $recurso, $param1, $param2, $param3) = split(';', $linea);
        print STDERR "[$$] type [$tipo] resource [$recurso]\n";
        if ($tipo eq 'art') {
            # Reduce recurso a un dia especifico.
            $recurso = substr($recurso,0,8);
            $$days{$recurso} = 1;
        } elsif ($tipo eq 'port') {
            $$days{'port'} = 1;
        }
    }
    close $FILE;

    unlink $file;
}

# ----------------------------------------------------------------------------- #
# Sincroniza los directorios de portadas completos.
sub procesar_port {
    &recurseDirs("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/edic",
                                            "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/edic");
}

# ----------------------------------------------------------------------------- #
# Sincroniza el directorio del dia del articulo completo.
sub procesa_art {
    my $ts = $_[0];
    my $dir_fecha;

    if ($ts =~ /(\d{8})/) {
        $dir_fecha = $1; # solo fecha.
        &recurseDirs("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dir_fecha",
                                                "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_ARTIC/$dir_fecha");
        # Se incluye la media externa.
        &recurseDirs("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EXMEDIA/$dir_fecha",
                                                "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}$prontus_varglb::DIR_EXMEDIA/$dir_fecha");
    }
}

# ----------------------------------------------------------------------------- #
# Sincroniza el directorio de la taxonomia.
sub procesa_tax {
    # Incluye taxonomia.
    &recurseDirs("$prontus_varglb::DIR_SERVER$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/cache/taxonomia/pags",
                "$prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DIR_SITE'}/cache/taxonomia/pags");
}

# ----------------------------------------------------------------------------- #
# Recorre los directorios recursivamente, subiendo a Dropbox los archivos que no
# existen o que tienen un peso distinto.
sub recurseDirs {
    my($source,$destination) = @_;
    my($path,$esdir,$size,$pid,$res);
    my ($fileTime,$numEntries,$aux,$localHash,$dropBoxHash,$debeSubir);
    my(@entries,@dropboxEntries);
    my %dropboxFileSizes;
    my %dropboxFileTimes;
    my %dropboxFileHashes;

    # Si el directorio es cpan/data/search o es /cache o /bak, sale sin hacer nada.
    if (($source =~ /cpan\/data\/search/) || ($source =~ /\/cache$/) || ($source =~ /\/bak$/)) {
        # print "    Banned Dir: $source\n"; # debug
        return;
    }

    opendir(DIR, $source) || die "Can't opendir " . $source . $!;
    @entries = readdir(DIR);
    closedir DIR;

    # Si el directorio esta vacio, sale sin hacer nada.
    $numEntries = scalar(@entries);
    if ($numEntries <= 2) {
        # print "    Empty Dir: $source\n"; # debug
        return;
    }

    print STDERR "[$$] Start: $source [$numEntries] -> $destination\n"; # debug

    # Lee las entradas de dropbox en el destino. <path>\t<es dir>\t<tamano en bytes>
    @dropboxEntries = lib_dropbox::getDir($destination);
    if ($lib_dropbox::ERROR) {
        # intentamos regenerar el token on the fly.
        if ($lib_dropbox::ERROR =~ /401 Unauthorized.*?invalid_access_token/) {
            $lib_dropbox::ATOKEN = '';
            $lib_dropbox::ATOKEN = lib_dropbox::getAccessToken($prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CLIENT_ID'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_CLIENT_SECRET'}, $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'DROPBOX_REFRESH_TOKEN'});
            print STDERR "Auth token: $lib_dropbox::ATOKEN\n";
            if (!$lib_dropbox::ATOKEN) {
                return;
            }
        } else {
            return;
        }
    }
    # Obtiene los tamanos de archivo. Si es un directorio asigna el tamano -1.
    %dropboxFileSizes = ();
    %dropboxFileTimes = ();
    # print "    Listado de $destination en dropbox:\n";
    foreach my $dropBoxEntry (@dropboxEntries) {
        ($path,$esdir,$size,$fileTime,$dropBoxHash) = split(/\t/,$dropBoxEntry);
        # Elimina los directorios superiores del path.
        if ($path =~ /\/([^\/]+)$/) {
            $path = $1;
        } else {
            $path = '/';
        }
        # Asigna el tamano.
        if ($esdir) {
            $dropboxFileSizes{$path} = -1;
        } else {
            $dropboxFileSizes{$path} = $size;
        }
        # Asigna el tiempo de modificacion en Dropbox.
        $dropboxFileTimes{$path} = glib_hrfec_03::ISO8601ToEpoch($fileTime);
        $dropboxFileHashes{$path} = $dropBoxHash;
        # print "    $path $size $fileTime\n"; # debug
    }

    # Recorre los archivos y subdirectorios.
    foreach my $entry (sort @entries) {
        # Si es un directorio, entonces aplica de nuevo la rutina en forma recursiva.
        # No copia los directorios que empiezan con un punto.
        if ((-d "$source/$entry") && ($entry !~ /^\./)) {
            &recurseDirs("$source/$entry","$destination/$entry");
        }

        if (-f "$source/$entry") {
            # No considera los archivos que contengan 'preview.' en su nombre,
            # o sean nombres baneados por dropbox.
            $aux = lc $entry;
            if (($aux =~ /preview\./) || ($aux eq 'desktop.ini') || ($aux eq 'thumbs.db')
                 || ($aux eq '.ds_store') || ($aux eq "icon\r") || ($aux =~ /^\.dropbox/)) {
                # print "  Me salto preview: $source/$entry\n";
                next;
            }
            # Si los tamanos son cero arriba y abajo, no hace nada.
            if (($dropboxFileSizes{$entry} == 0) && ((-s "$source/$entry") == 0)) {
                next;
            }
            $debeSubir = 0;
            # Si no existe en dropbox o si los tamanos son distintos, lo sube.
            if ( (! $dropboxFileSizes{$entry}) || ( $dropboxFileSizes{$entry} != -s "$source/$entry") ) {
                $debeSubir = 1;
            }
            ($dropBoxHash,$localHash) = ('','');
            # Si la fecha local es mas reciente que en dropbox, verifica hashes y sube si corresponde.
            if (($debeSubir == 0) && ( $dropboxFileTimes{$entry} < (stat "$source/$entry")[9] )) {
                # Compara los hashes por si los contenidos son identicos (suele suceder...).
                $dropBoxHash = $dropboxFileHashes{$entry};
                $localHash = lib_dropbox::computeContentHash("$source/$entry");
                if ($localHash ne $dropBoxHash) {
                    $debeSubir = 1;
                }
            }
            if ($debeSubir == 1) {
                print STDERR "[$$] Sube: $source/$entry d[", $dropboxFileSizes{$entry};
                print STDERR "] s[", (-s "$source/$entry") , "] t[" ,((stat "$source/$entry")[9] - $dropboxFileTimes{$entry});
                print STDERR "] d[$dropBoxHash] s[$localHash]\n"; # debug
                # Hace la sincronizacion.
                $res = 0;
                while ($res == 0) {
                    $res = lib_dropbox::putFile("$source/$entry","$destination/$entry");
                    if ($res < 0) {
                        print STDERR lib_lang::get_interpolated_message('dropbox_backup_upload_fail', $$, $res, "$source/$entry")."\n"; # debug
                        if ($res == -3) {
                            print STDERR "[$$] Falló la autenticación con Dropbox. Reintentar todo.\n";
                            exit;
                        }
                    } else {
                        print STDERR "[$$] Upload OK [$res]: $source/$entry\n"; # debug
                    }
                }
            }
        }
    }
    if ($lib_dropbox::ERROR ne '') {
        &enviar_mail_error();
    }
}

# ----------------------------------------------------------------------------- #
sub epochToFechaHora {
    my $epoch = $_[0];
    my @utc = gmtime($epoch);
    my $fechahora = sprintf('%04d/%02d/%02d %02d:%02d:%02d',(1900 + $utc[5]),(1 + $utc[4]),$utc[3],$utc[2],$utc[1],$utc[0]);
}

# Envía un email con los errores encontrados
# @param string $email el destinatario
# @param string $texto el mensaje.
# @return el resultado de lib_mail::enviar_mail()
sub enviar_mail_error {
    my $email = 'area_prontus@altavoz.net';
    my $from = 'prontus@' . $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'PRONTUS_MAIL_DOMAIN'};
    my $replyto_name = 'Prontus CMS';
    my $replyto_email = $email;
    my $server = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'CPAN_SERVER_NAME'};
    if ($server =~ /^[0-9]+\.[0-9]+\./) {
        my $dir_server_ofusc = $prontus_varglb::DIR_SERVER;
        $dir_server_ofusc =~ s/\//-/g;
        $dir_server_ofusc =~ s/^[- ]+//g;
        $server .= " ($dir_server_ofusc)";
    }
    my $asunto = lib_lang::get_interpolated_message('dropbox_backup_email_subject_error', $server, $prontus_varglb::PRONTUS_ID);

    my $texto = $lib_dropbox::ERROR;
    my $htmldoc = '';
    my $attach = '';
    my $url = '';
    my $dir_attach = '';
    my $smtp = $prontus_varglb::CONFIG{$prontus_varglb::PRONTUS_ID}{'SERVER_SMTP'};

    lib_mail::enviar_mail($email, $from, $replyto_name, $replyto_email, $asunto, $texto, $htmldoc, $attach, $url, $dir_attach, $smtp);
}
